﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Office.Interop.Word;
using System.Reflection;

[assembly: CLSCompliant(true)]
namespace Core
{
    /// <summary>
    /// Maintain a single instance of Word application during the life cycle of the project application. 
    /// After the instance is accessed, this class will hold Word Application forever.
    /// </summary>
    internal sealed class WordSingleton
    {
        #region Singleton
        WordSingleton()
        {
            System.Diagnostics.Debug.WriteLine("Singleton is created.");
        }

        /// <summary>
        /// Provide the single instance.
        /// </summary>
        public static WordSingleton Instance
        {
            get
            {
                return Nested.instance;
            }
        }

        class Nested
        {
            static Nested() { }

            internal static readonly WordSingleton instance = new WordSingleton();
        }
        #endregion

        ApplicationClass wordApplication;

        bool dirty;

        /// <summary>
        /// True if a Word application object is created.
        /// False if the Word application object is not yet created, or is closed.
        /// </summary>
        public bool Dirty { get { return dirty; } }

        /// <summary>
        /// Return a word application object which is created only when accessed for the first time.
        /// </summary>
        public _Application WordApplication
        {
            get
            {
                if (wordApplication == null)
                {
                    wordApplication = new Microsoft.Office.Interop.Word.ApplicationClass();
                    
                    dirty = true;
                    System.Diagnostics.Debug.WriteLine("Word application is created.");

                    wordApplication.ApplicationEvents4_Event_Quit += new ApplicationEvents4_QuitEventHandler(wordApplication_ApplicationEvents4_Event_Quit);
                }

                return wordApplication;
            }

        }

        void wordApplication_ApplicationEvents4_Event_Quit()
        {
            wordApplication = null;
            dirty = false;
            System.Diagnostics.Debug.WriteLine("Word aplication is closed by end user.");
        }
    }

    /// <summary>
    /// Provide / brorrow functions of Microsoft Word. 
    /// A single instance of Word document
    /// may persist throughout the life cycle of the project application in order to
    /// get performance.
    /// </summary>
    public sealed class WordFunctions
    {
        private WordFunctions() { }

        /// <summary>
        /// Quit Word application if it was created.
        /// </summary>
        public static void CloseWordApplication()
        {
            if (WordSingleton.Instance.Dirty)
            {
                object saveChange = 0;
                object originalFormat = Missing.Value;
                object routeDocument = Missing.Value;

                try
                {
                    WordSingleton.Instance.WordApplication.Quit(ref saveChange, ref originalFormat, ref routeDocument);
                    System.Diagnostics.Debug.WriteLine("Word Application quit.");
                }
                catch (System.Runtime.InteropServices.COMException ex)
                {
                    System.Diagnostics.Trace.WriteLine(ex.Message);
                }

            }
        }
        /// <summary>
        /// Convert from Simplified Chinese to Traditional Chinese.
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static string CHS2CHT(string text)
        {
            Document doc = WordDocumentSingleton.Instance.Document;// or WordDocumentSingleton.Instance.Document;
            doc.SpellingChecked = false;
            doc.Content.Text = text;
            doc.Content.TCSCConverter(WdTCSCConverterDirection.wdTCSCConverterDirectionSCTC, true, true);
            return doc.Content.Text;
        }

        /// <summary>
        /// Convert from Traditional Chinese to Simplified Chinese.
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static string CHT2CHS(string text)
        {
            Document doc = WordDocumentSingleton.Instance.Document;// or WordDocumentSingleton.Instance.Document;
            doc.SpellingChecked = false;
            doc.Content.Text = text;
            doc.Content.TCSCConverter(WdTCSCConverterDirection.wdTCSCConverterDirectionTCSC, true, true);
            return doc.Content.Text;
        }

        /// <summary>
        /// Covert a word document and save it as another file.
        /// </summary>
        /// <param name="sourceName"></param>
        /// <param name="targetName"></param>
        /// <param name="ChsToCht">True for converting from Chs to Cht.</param>
        public static void ConvertWordDocument(string sourceName, string targetName, bool ChsToCht)
        {
            object fileName = sourceName;
            object dummy = Missing.Value;
            _Document doc = WordSingleton.Instance.WordApplication.Documents.Open(ref fileName, ref dummy, ref dummy,
                ref dummy, ref dummy,ref dummy,ref dummy,ref dummy,ref dummy,ref dummy,ref dummy,ref dummy,ref dummy,ref dummy,ref dummy, ref dummy);

            if (ChsToCht)
            {
                doc.Content.TCSCConverter(WdTCSCConverterDirection.wdTCSCConverterDirectionSCTC, true, true);
            }
            else
            {
                doc.Content.TCSCConverter(WdTCSCConverterDirection.wdTCSCConverterDirectionTCSC, true, true);
            }

            fileName = targetName;
            doc.SaveAs(ref fileName, ref dummy, ref dummy, ref dummy, ref dummy, ref dummy, ref dummy, ref dummy, ref dummy,
                ref dummy, ref dummy, ref dummy, ref dummy, ref dummy, ref dummy, ref dummy);
            doc.Close(ref dummy, ref dummy, ref dummy);
        }
    }

    /// <summary>
    /// Hold a invisible document during the life cycle of project application.
    /// </summary>
    internal sealed class WordDocumentSingleton
    {
        #region Singleton
        WordDocumentSingleton()
        { }

        public static WordDocumentSingleton Instance
        {
            get { return Nested.instance; }
        }

        class Nested
        {
            static Nested() { }

            internal static readonly WordDocumentSingleton instance = new WordDocumentSingleton();

        }
        #endregion

        Document document;

        public Document Document
        {
            get
            {
                if (document == null)
                {
                    object template = Missing.Value;
                    object newTemplate = Missing.Value;
                    object docType = Missing.Value;
                    object visible = false;  // must be true to have Selection available.
                    document = WordSingleton.Instance.WordApplication.Documents.Add(ref template, ref newTemplate, ref docType, ref visible);
                }

                return document;
            }
        }

    }
}
